package javax.swing;

import javax.accessibility.AccessibleRole;
import javax.accessibility.AccessibleContext;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.util.Objects;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.ToolTipUI;
import javax.accessibility.Accessible;

public class JToolTip extends JComponent implements Accessible
{
    private static final String uiClassID = "ToolTipUI";
    String tipText;
    JComponent component;
    
    public JToolTip() {
        this.setOpaque(true);
        this.updateUI();
    }
    
    public ToolTipUI getUI() {
        return (ToolTipUI)this.ui;
    }
    
    @Override
    public void updateUI() {
        this.setUI((ComponentUI)UIManager.getUI(this));
    }
    
    @Override
    public String getUIClassID() {
        return "ToolTipUI";
    }
    
    public void setTipText(String s) {
        String tipText = this.tipText;
        this.firePropertyChange("tiptext", tipText, this.tipText = s);
        if (!Objects.equals(tipText, s)) {
            this.revalidate();
            this.repaint();
        }
    }
    
    public String getTipText() {
        return this.tipText;
    }
    
    public void setComponent(JComponent component) {
        this.firePropertyChange("component", this.component, this.component = component);
    }
    
    public JComponent getComponent() {
        return this.component;
    }
    
    @Override
    boolean alwaysOnTop() {
        return true;
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.defaultWriteObject();
        if (this.getUIClassID().equals("ToolTipUI")) {
            int n = (byte)(JComponent.getWriteObjCounter(this) - 1);
            JComponent.setWriteObjCounter(this, (byte)n);
            if (n == 0 && this.ui != null) {
                this.ui.installUI(this);
            }
        }
    }
    
    @Override
    protected String paramString() {
        return super.paramString() + ",tipText=" + ((this.tipText != null) ? this.tipText : "");
    }
    
    @Override
    public AccessibleContext getAccessibleContext() {
        if (this.accessibleContext == null) {
            this.accessibleContext = new JToolTip.AccessibleJToolTip();
        }
        return this.accessibleContext;
    }
    
    protected class AccessibleJToolTip extends JComponent.AccessibleJComponent
    {
        @Override
        public String getAccessibleDescription() {
            String s = this.accessibleDescription;
            if (s == null) {
                s = (String)JToolTip.this.getClientProperty("AccessibleDescription");
            }
            if (s == null) {
                s = JToolTip.this.getTipText();
                return s;
            }
            return s;
        }
        
        @Override
        public AccessibleRole getAccessibleRole() {
            return AccessibleRole.TOOL_TIP;
        }
    }
}
